@@ -6,20 +6,39 @@
 #' @param rle.depth s Numeric. Ceiling for number of rows of run length encoding. Fewer rows means longer runs.
 #' @param case.sensitive Logical.
 #' @param multi.core Logical or Numeric. \code{TRUE} uses \code{parallel::detectCores()}. \code{FALSE} uses one, single core. You can also specify the number logical cores. Mac and Unix only.
+#' @param dev.mode Logical. Development mode uses parallel::parLapply().
 #' @export
 
 ipFilter <- function(cran_log, campaigns = TRUE, rle.depth = 100,
-  case.sensitive = FALSE, multi.core = TRUE) {
+  case.sensitive = FALSE, multi.core = TRUE, dev.mode = dev.mode) {
 
   cores <- multiCore(multi.core)
+  # win.exception <- .Platform$OS.type == "windows" & cores > 1
+
   greedy.ips <- ip_filter(cran_log)
 
   if (campaigns) {
-    candidate.data <- parallel::mclapply(greedy.ips$package.ip, function(ip) {
-      tmp <- cran_log[cran_log$ip_id == ip, ]
-      tmp$t2 <- as.POSIXlt(paste(tmp$date, tmp$time), tz = "GMT")
-      tmp[order(tmp$t2, tmp$package), ]
-    }, mc.cores = cores)
+    # if (dev.mode | win.exception) {
+    if (dev.mode) {
+      cl <- parallel::makeCluster(cores)
+      parallel::clusterExport(cl = cl, envir = environment(),
+        varlist = c("greedy.ips", "cran_log"))
+      candidate.data <- parallel::parLapply(cl, greedy.ips$package.ip,
+        function(ip) {
+          tmp <- cran_log[cran_log$ip_id == ip, ]
+          tmp$t2 <- as.POSIXlt(paste(tmp$date, tmp$time), tz = "GMT")
+          tmp[order(tmp$t2, tmp$package), ]
+      })
+      parallel::stopCluster(cl)
+
+    } else {
+      if (.Platform$OS.type == "windows") cores <- 1L
+      candidate.data <- parallel::mclapply(greedy.ips$package.ip, function(ip) {
+        tmp <- cran_log[cran_log$ip_id == ip, ]
+        tmp$t2 <- as.POSIXlt(paste(tmp$date, tmp$time), tz = "GMT")
+        tmp[order(tmp$t2, tmp$package), ]
+      }, mc.cores = cores)
+    }
 
     rle.data <- lapply(candidate.data, function(x) {
       runLengthEncoding(x, case.sensitive = case.sensitive)
@@ -30,41 +49,70 @@ ipFilter <- function(cran_log, campaigns = TRUE, rle.depth = 100,
 
     # check for campaigns #
 
-    campaign.row.delete <- parallel::mclapply(candidate.ids, function(x) {
-      tmp <- rle.data[[x]]
-      A <- tmp[tmp$letter == "a" & tmp$lengths >= 10, ]
-      start <- as.numeric(row.names(A))
-      end <- as.numeric(row.names(A)) + length(letters) - 1
+    # if (dev.mode | win.exception) {
+    if (dev.mode) {
+      cl <- parallel::makeCluster(cores)
 
-      data.select <- lapply(seq_along(start), function(i) {
-        audit.data <- tmp[start[i]:end[i], ]
-        if (all(!is.na(audit.data))) {
-          data.frame(ip = greedy.ips$package.ip[x],
-                     start = audit.data[1, "start"],
-                     end = audit.data[nrow(audit.data), "end"])
-        }
+      parallel::clusterExport(cl = cl, envir = environment(),
+        varlist = c("candidate.ids", "rle.data", "candidate.data"))
+
+      campaign.row.delete <- parallel::parLapply(cl, candidate.ids,
+        function(x) {
+        tmp <- rle.data[[x]]
+        A <- tmp[tmp$letter == "a" & tmp$lengths >= 10, ]
+        start <- as.numeric(row.names(A))
+        end <- as.numeric(row.names(A)) + length(letters) - 1
+        data.select <- lapply(seq_along(start), function(i) {
+          audit.data <- tmp[start[i]:end[i], ]
+          if (all(!is.na(audit.data))) {
+            data.frame(ip = greedy.ips$package.ip[x],
+                       start = audit.data[1, "start"],
+                       end = audit.data[nrow(audit.data), "end"])
+          }
+        })
+        data.select <- do.call(rbind, data.select)
+        c.data <- candidate.data[[x]]
+        if (!is.null(data.select)) {
+          unlist(lapply(seq_len(nrow(data.select)), function(i) {
+            row.names(c.data[data.select[i, "start"]:data.select[i, "end"], ])
+          }))
+        } else NULL
       })
 
-      data.select <- do.call(rbind, data.select)
-      c.data <- candidate.data[[x]]
-
-      if (!is.null(data.select)) {
-        unlist(lapply(seq_len(nrow(data.select)), function(i) {
-          row.names(c.data[data.select[i, "start"]:data.select[i, "end"], ])
-        }))
-      } else NULL
-    }, mc.cores = cores)
+      parallel::stopCluster(cl)
 
+    } else {
+      if (.Platform$OS.type == "windows") cores <- 1L
+      campaign.row.delete <- parallel::mclapply(candidate.ids, function(x) {
+        tmp <- rle.data[[x]]
+        A <- tmp[tmp$letter == "a" & tmp$lengths >= 10, ]
+        start <- as.numeric(row.names(A))
+        end <- as.numeric(row.names(A)) + length(letters) - 1
+        data.select <- lapply(seq_along(start), function(i) {
+          audit.data <- tmp[start[i]:end[i], ]
+          if (all(!is.na(audit.data))) {
+            data.frame(ip = greedy.ips$package.ip[x],
+                       start = audit.data[1, "start"],
+                       end = audit.data[nrow(audit.data), "end"])
+          }
+        })
+        data.select <- do.call(rbind, data.select)
+        c.data <- candidate.data[[x]]
+        if (!is.null(data.select)) {
+          unlist(lapply(seq_len(nrow(data.select)), function(i) {
+            row.names(c.data[data.select[i, "start"]:data.select[i, "end"], ])
+          }))
+        } else NULL
+      }, mc.cores = cores)
+    }
     sel <- cran_log$ip_id %in% greedy.ips$ratio.ip
     ratio.row.delete <- row.names(cran_log[sel, ])
     rows.delete <- c(unlist(campaign.row.delete), ratio.row.delete)
-
   } else {
     sel <- cran_log$ip_id %in% unlist(greedy.ips)
     rows.delete <- row.names(cran_log[sel, ])
   }
-
-  rows.delete
+  cran_log[!row.names(cran_log) %in% rows.delete, ]
 }
 
 #' Identify IP's that are mirroring CRAN (k-means helper prototype).
